@@ -36,14 +36,14 @@ public:
         // one to use. If it turns out that the two will never be installed at the
         // same time, then this perhaps should be "signon" instead, which then would
         // pick either a gSSO or UAO backend depending on which is available.
-#if defined(USE_GSSO) || defined(STATIC_GSSO)
+#if defined(USE_ACCOUNTS) && defined(USE_GSSO) || defined(STATIC_GSSO)
         IdentityProvider("gsso",
                          "gsso:<numeric account ID>[,<service name>]\n"
                          "   Authentication using libgsignond + libaccounts,\n"
                          "   using an account created and managed with libaccounts.\n"
                          "   The service name is optional. If not given, the\n"
                          "   settings from the account will be used.")
-#elif defined(USE_UOA) || defined(STATIC_UOA)
+#elif defined(USE_ACCOUNTS) && defined(USE_UOA) || defined(STATIC_UOA)
         IdentityProvider("uoa",
                          "uoa:<numeric account ID>[,<service name>]\n"
                          "   Authentication using libsignon + libaccounts,\n"
