@@ -53,6 +53,12 @@ class ConfigNode {
     /** a name for the node that the user can understand */
     virtual std::string getName() const = 0;
 
+    /**
+     * True if the config node has no persistent storage behind it.
+     * In that case, flush() and setProperty() may as well be skipped.
+     */
+    virtual bool isVolatile() const = 0;
+
     /**
      * save all changes persistently
      */
