@@ -19,7 +19,7 @@
 
 #define EDS_ABI_WRAPPER_NO_REDEFINE 1
 #include <syncevo/eds_abi_wrapper.h>
-#include <syncevo/SyncContext.h>
+#include <syncevo/util.h>
 
 #include <string>
 #include <sstream>
@@ -30,7 +30,8 @@
 using namespace std;
 namespace {
 
-std::string lookupDebug, lookupInfo;
+static std::string &getLookupDebug() { static std::string lookupDebug; return lookupDebug; }
+static std::string &getLookupInfo() { static std::string lookupInfo; return lookupInfo; }
 
 }
 
@@ -114,10 +115,15 @@ void *findSymbols(const char *libname, int minver, int maxver,
         const char *symname = NULL;
         while (funcptr) {
             symname = va_arg(ap, const char *);
-            *funcptr = dlsym(dlhandle, symname);
+            // Don't overwrite address already found earlier. Needed
+            // for icaltimezone_get_component(), which may be
+            // overridden by ourselves.
             if (!*funcptr) {
-                debug << symname << " not found" << std::endl;
-                allfound = false;
+                *funcptr = dlsym(dlhandle, symname);
+                if (!*funcptr) {
+                    debug << symname << " not found" << std::endl;
+                    allfound = false;
+                }
             }
             funcptr = va_arg(ap, void **);
         }
@@ -140,9 +146,9 @@ void *findSymbols(const char *libname, int minver, int maxver,
         }
     }
 
-    lookupInfo += info.str();
-    lookupDebug += info.str();
-    lookupDebug += debug.str();
+    getLookupInfo() += info.str();
+    getLookupDebug() += info.str();
+    getLookupDebug() += debug.str();
     return dlhandle;
 }
 
@@ -178,6 +184,14 @@ extern "C" void EDSAbiWrapperInit()
         initialized = true;
     }
 
+    // Bind icaltimezone_get_component() to the version found (or not found,
+    // if internal-icaltz not configured) in our own executable.
+    void *get = dlsym(RTLD_DEFAULT, "icaltimezone_get_component");
+    getLookupDebug() += SyncEvo::StringPrintf("icaltimezone_get_component = %p", get);
+#if defined(ENABLE_ECAL) || defined(ENABLE_ICAL)
+    EDSAbiWrapperSingleton.icaltimezone_get_component = (icalcomponent *(*)(icaltimezone *))get;
+#endif
+
 # ifdef HAVE_EDS
     edshandle =
     findSymbols("libedataserver-1.2.so", 7, 16,
@@ -248,6 +262,7 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.icalcomponent_get_summary, "icalcomponent_get_summary", \
                 &EDSAbiWrapperSingleton.icalcomponent_get_uid, "icalcomponent_get_uid", \
                 &EDSAbiWrapperSingleton.icalcomponent_get_dtstart, "icalcomponent_get_dtstart", \
+                &EDSAbiWrapperSingleton.icalcomponent_set_dtstart, "icalcomponent_set_dtstart", \
                 &EDSAbiWrapperSingleton.icalcomponent_isa, "icalcomponent_isa", \
                 &EDSAbiWrapperSingleton.icalcomponent_new_clone, "icalcomponent_new_clone", \
                 &EDSAbiWrapperSingleton.icalcomponent_new_from_string, "icalcomponent_new_from_string", \
@@ -306,7 +321,26 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.icaltimezone_get_component, "icaltimezone_get_component", \
                 &EDSAbiWrapperSingleton.icaltimezone_get_tzid, "icaltimezone_get_tzid", \
                 &EDSAbiWrapperSingleton.icaltimezone_new, "icaltimezone_new", \
-                &EDSAbiWrapperSingleton.icaltimezone_set_component, "icaltimezone_set_component",
+                &EDSAbiWrapperSingleton.icaltimezone_set_component, "icaltimezone_set_component", \
+                &EDSAbiWrapperSingleton.icaltimezone_get_location, "icaltimezone_get_location", \
+                &EDSAbiWrapperSingleton.ical_tzid_prefix, "ical_tzid_prefix", \
+                &EDSAbiWrapperSingleton.icalerror_set_errno, "icalerror_set_errno", \
+                &EDSAbiWrapperSingleton.icalproperty_new_dtstart, "icalproperty_new_dtstart", \
+                &EDSAbiWrapperSingleton.icalproperty_new_rrule, "icalproperty_new_rrule", \
+                &EDSAbiWrapperSingleton.icalproperty_new_tzid, "icalproperty_new_tzid", \
+                &EDSAbiWrapperSingleton.icalproperty_new_tzname, "icalproperty_new_tzname", \
+                &EDSAbiWrapperSingleton.icalproperty_new_tzoffsetfrom, "icalproperty_new_tzoffsetfrom", \
+                &EDSAbiWrapperSingleton.icalproperty_new_tzoffsetto, "icalproperty_new_tzoffsetto", \
+                &EDSAbiWrapperSingleton.icalproperty_new_x, "icalproperty_new_x", \
+                &EDSAbiWrapperSingleton.icalproperty_set_x_name, "icalproperty_set_x_name", \
+                &EDSAbiWrapperSingleton.icalrecur_iterator_free, "icalrecur_iterator_free", \
+                &EDSAbiWrapperSingleton.icalrecur_iterator_new, "icalrecur_iterator_new", \
+                &EDSAbiWrapperSingleton.icalrecur_iterator_next, "icalrecur_iterator_next", \
+                &EDSAbiWrapperSingleton.icalrecurrencetype_clear, "icalrecurrencetype_clear", \
+                &EDSAbiWrapperSingleton.icaltime_day_of_week, "icaltime_day_of_week", \
+                &EDSAbiWrapperSingleton.icaltime_days_in_month, "icaltime_days_in_month", \
+                &EDSAbiWrapperSingleton.icaltzutil_get_zone_directory, "icaltzutil_get_zone_directory", \
+
 
     // icalparameter_new_scheduleagent was added in libical.so.1. We
     // use it only to detect the libical 1.0 ABI. This works because
@@ -433,20 +467,30 @@ void EDSAbiWrapperInit()
         initialized = true;
     }
 
+    // Bind icaltimezone_get_component() and
+    // icaltzutil_fetch_timezone() to the version found (or not found,
+    // if not enabled) in our own executable. Without this, the
+    // dynamic linker on Ubuntu Saucy and Trusty prefers the versions
+    // from libical once it is loaded.
+    void *fetch = dlsym(RTLD_DEFAULT, "icaltzutil_fetch_timezone");
+    void *get = dlsym(RTLD_DEFAULT, "icaltimezone_get_component");
+    getLookupDebug() += SyncEvo::StringPrintf("icaltzutil_fetch_timezone = %p\n", fetch);
+    getLookupDebug() += SyncEvo::StringPrintf("icaltimezone_get_component = %p", get);
+
     static const char *soname = "libical.so.1";
     void *dlhandle = dlopen(soname, RTLD_GLOBAL|RTLD_LAZY);
     if (dlhandle) {
-        lookupInfo += "using ";
-        lookupInfo += soname;
-        lookupInfo += "\n";
+        getLookupInfo() += "using ";
+        getLookupInfo() += soname;
+        getLookupInfo() += "\n";
         EDSAbiHaveIcal1 = 1;
         dlclose(dlhandle);
     }
 }
 #endif // EVOLUTION_COMPATIBILITY
 
-extern "C" const char *EDSAbiWrapperInfo() { EDSAbiWrapperInit(); return lookupInfo.c_str(); }
-extern "C" const char *EDSAbiWrapperDebug() { EDSAbiWrapperInit(); return lookupDebug.c_str(); }
+extern "C" const char *EDSAbiWrapperInfo() { EDSAbiWrapperInit(); return getLookupInfo().c_str(); }
+extern "C" const char *EDSAbiWrapperDebug() { EDSAbiWrapperInit(); return getLookupDebug().c_str(); }
 
 #ifdef ENABLE_DBUS_TIMEOUT_HACK
 /**
