@@ -4,12 +4,14 @@
 
 set -ex
 
-TMPFILE=`mktemp`
-TMPFILE_CXX=`mktemp`.cxx
-TMPFILE_O=`mktemp`.o
+DIR=`mktemp -d`
+TMPFILE_CXX=$DIR/installcheck-local.cpp
+TMPFILE_O=$DIR/installcheck-local.o
+TMPFILE=$DIR/installcheck-local
 
 rmtmp () {
     rm -f $TMPFILE $TMPFILE_CXX $TMPFILE_O
+    rmdir $DIR
 }
 trap rmtmp EXIT
 
@@ -51,4 +53,4 @@ done
 # must take DESTDIR into account by adding -L<libdir> (skipped when equal to /usr/lib)
 # and modifying any additional paths including that
 pkg-config --libs syncevolution
-env LD_LIBRARY_PATH=$3:$3/syncevolution $LD_LIBRARY_PATH $CXX -v $TMPFILE_O -o $TMPFILE "-L$3" `pkg-config --libs syncevolution | sed -e "s;/usr/lib;$3;g"`
+env LD_LIBRARY_PATH=$3:$3/syncevolution:$LD_LIBRARY_PATH $CXX -v $TMPFILE_O -o $TMPFILE "-L$3" `pkg-config --libs syncevolution | sed -e "s;/usr/lib;$3;g"`
