@@ -23,8 +23,27 @@
 #include "platform_thread.h"
 #endif
 
+#ifdef USE_DLT
+#include <dlt.h>
+#endif
+
 namespace sysync {
 
+#ifdef USE_DLT
+static bool DbgDLTInitialized;
+static DltContext DbgProtoContext;
+static DltContext DbgSessionContext;
+static DltContext DbgAdminContext;
+static DltContext DbgDataContext;
+static DltContext DbgRemoteInfoContext;
+static DltContext DbgParseContext;
+static DltContext DbgGenerateContext;
+static DltContext DbgTranspContext;
+static DltContext DbgSyncMLTKContext;
+static DltContext DbgDefaultContext;
+#endif
+
+
 #ifndef HARDCODED_CONFIG
 
 // debug format modes
@@ -32,6 +51,13 @@ cAppCharP const DbgOutFormatNames[numDbgOutFormats] = {
   "text",       // plain text format (but can be indented)
   "xml",        // XML format
   "html"        // HTML format
+#ifdef USE_DLT
+  // If DLT support is not enabled, then trying to uses it in a config
+  // will lead to a generic parse error. Might be good enough, although
+  // a dedicated error message about "DLT being disabled in this build"
+  // would be nicer.
+  , "dlt"         // GENIVI Diagnostic Log and Trace
+#endif
 };
 
 
@@ -83,7 +109,7 @@ cAppCharP const DbgOutDefaultPrefixes[numDbgOutFormats] = {
   "<?xml version=\"1.0\"?>\n"
     "<sysync_log version=\"1.0\">",
   "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n"
-  "<html><head><title>Synthesis SyncML Engine " SYSYNC_FULL_VERSION_STRING " Log</title>\n"
+  "<html><head><title>SySync SyncML Engine " SYSYNC_FULL_VERSION_STRING " Log</title>\n"
     "<meta http-equiv=\"content-type\" content=\"text/html;charset=UTF-8\">\n"
     "<style type=\"text/css\" media=\"screen\"><!--\n"
       ".block { color: #0000FF; font-weight: bold; }\n"
@@ -141,7 +167,7 @@ cAppCharP const DbgOutDefaultPrefixes[numDbgOutFormats] = {
       "}\n"
       ".doall { color: #754242; }\n"
     "--></style>\n"
-    "</head><body><h2>Start of log - Synthesis SyncML Engine " SYSYNC_FULL_VERSION_STRING "</h2>\n<ul>\n"
+    "</head><body><h2>Start of log - SySync SyncML Engine " SYSYNC_FULL_VERSION_STRING "</h2>\n<ul>\n"
 };
 
 cAppCharP const DbgOutDefaultSuffixes[numDbgOutFormats] = {
@@ -298,6 +324,28 @@ TStdFileDbgOut::~TStdFileDbgOut()
 } // TStdFileDbgOut::~TStdFileDbgOut
 
 
+#ifdef LINUX
+# include <unistd.h>
+# include <fcntl.h>
+#endif
+
+static FILE *FOpen(const char *path, const char *mode)
+{
+  FILE *file = fopen(path, mode);
+#ifdef LINUX
+  // Be careful not to leak this file descriptor into forked
+  // processes.
+  int fd = file ? fileno(file) : -1;
+  if (fd >= 0) {
+    int flags = fcntl(fd, F_GETFD);
+    if (flags != -1) {
+      fcntl(fd, F_SETFD, flags | FD_CLOEXEC);
+    }
+  }
+#endif
+  return file;
+}
+
 // open standard C file based debug output channel
 bool TStdFileDbgOut::openDbg(cAppCharP aDbgOutputName, cAppCharP aSuggestedExtension, TDbgFlushModes aFlushMode, bool aOverWrite, bool aRawMode)
 {
@@ -312,7 +360,7 @@ bool TStdFileDbgOut::openDbg(cAppCharP aDbgOutputName, cAppCharP aSuggestedExten
   // for C files, use the extension provided
   fFileName+=aSuggestedExtension;
   // open
-  fFile=fopen(fFileName.c_str(),aRawMode ? (aOverWrite ? "wb" : "ab") : (aOverWrite ? "w" : "a"));
+  fFile=FOpen(fFileName.c_str(),aRawMode ? (aOverWrite ? "wb" : "ab") : (aOverWrite ? "w" : "a"));
   // in case this fails, we'll have a NULL fFile. We can't do anything more here
   fIsOpen=fFile!=NULL;
   // For openclose mode, we have opened here only to check for logfile writability - close again
@@ -332,7 +380,7 @@ uInt32 TStdFileDbgOut::dbgFileSize(void)
   uInt32 sz;
   if (fFlushMode==dbgflush_openclose) {
     // we need to open the file for append first
-    fFile=fopen(fFileName.c_str(),"a");
+    fFile=FOpen(fFileName.c_str(),"a");
     fseek(fFile,0,SEEK_END); // move to end (needed, otherwise ftell may return 0 despite "a" fopen mode)
     sz=ftell(fFile);
     fclose(fFile);
@@ -367,7 +415,7 @@ void TStdFileDbgOut::putLine(cAppCharP aLine, bool aForceFlush)
     if (fFlushMode==dbgflush_openclose) {
       // we need to open the file for append first
       lockMutex(mutex);
-      fFile=fopen(fFileName.c_str(),"a");
+      fFile=FOpen(fFileName.c_str(),"a");
       if (!fFile)
         unlockMutex(mutex);
     }
@@ -399,7 +447,7 @@ void TStdFileDbgOut::putRawData(cAppPointer aData, memSize aSize)
     if (fFlushMode==dbgflush_openclose) {
       // we need to open the file for append first
       lockMutex(mutex);
-      fFile=fopen(fFileName.c_str(),"a");
+      fFile=FOpen(fFileName.c_str(),"a");
     }
     if (fFile) {
       if (fwrite(aData, 1, aSize, fFile) != 1) {
@@ -482,6 +530,8 @@ TDebugLoggerBase::TDebugLoggerBase(GZones *aGZonesP) :
   fIndent=0;
   fBlockHistory=NULL; // no Block open yet
   fOutStarted=false; // not yet started
+  fBlockNo=0;
+  fGZonesP=NULL;
   fOutputLoggerP=NULL; // no redirected output yet
 } // TDebugLoggerBase::TDebugLoggerBase
 
@@ -731,6 +781,39 @@ string TDebugLoggerBase::dbg2Link(const TDbgLocation &aTDbgLoc, const string &aT
 
 #endif // SYDEBUG_LOCATION
 
+#ifdef USE_DLT
+static void DbgText2PlainText(const char *in, size_t len, std::string &out)
+{
+  const char *q=in;
+  const char *s=q;
+  const char *end=in + len;
+
+  while (q<end) {
+    if (*q=='&') {
+      if (end-q>=6 && strucmp(q,"&html;",6)==0) {
+        if (q>s) out.append(s,q-s);
+        // everything until next &html; must be filtered out
+        // - search next &html;
+        s=q=q+6;
+        while(q+6<=end && strucmp(q,"&html;",6)!=0) q++;
+        s=q=q+6;
+      }
+      else if (end-q>=4 && strucmp(q,"&sp;",4)==0) {
+        if (q>s) out.append(s,q-s);
+        s=q=q+4;
+        out += ' '; // convert to plain space
+      }
+      else
+        q++;
+    }
+    else {
+      q++;
+    }
+  }
+  if (q>s) out.append(s,q-s);
+}
+#endif // USE_DLT
+
 
 // output text to debug channel
 void TDebugLoggerBase::DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP aText, stringSize aTextSize, bool aPreFormatted)
@@ -757,6 +840,72 @@ void TDebugLoggerBase::DebugPuts(TDBG_LOCATION_PROTO uInt32 aDbgMask, cAppCharP
         return; // stop all efforts here
       }
     }
+
+#ifdef USE_DLT
+    // DLT logging logs everything in one chunk
+    if (fDbgOptionsP->fOutputFormat == dbgfmt_dlt) {
+      DltContext *context = &DbgDefaultContext;
+      if (aDbgMask & DBG_PROTO) {
+        context = &DbgProtoContext;
+      }
+      else if (aDbgMask & DBG_SESSION) {
+        context = &DbgSessionContext;
+      }
+      else if (aDbgMask & DBG_ADMIN) {
+        context = &DbgAdminContext;
+      }
+      else if (aDbgMask & DBG_DATA) {
+        context = &DbgDataContext;
+      }
+      else if (aDbgMask & DBG_REMOTEINFO) {
+        context = &DbgRemoteInfoContext;
+      }
+      else if (aDbgMask & DBG_PARSE) {
+        context = &DbgParseContext;
+      }
+      else if (aDbgMask & DBG_GEN) {
+        context = &DbgGenerateContext;
+      }
+      else if (aDbgMask & DBG_TRANSP) {
+        context = &DbgTranspContext;
+      }
+      else if (aDbgMask & (DBG_RTK_SML|DBG_RTK_XPT)) {
+        context = &DbgSyncMLTKContext;
+      }
+
+      DltLogLevelType level = DLT_LOG_VERBOSE;
+      if (level > DLT_LOG_INFO &&
+          (aDbgMask & DBG_HOT)) {
+        level = DLT_LOG_INFO;
+      }
+      if (level > DLT_LOG_ERROR &&
+          (aDbgMask & DBG_ERROR)) {
+        level = DLT_LOG_ERROR;
+      }
+      if (level > DLT_LOG_DEBUG &&
+          (aDbgMask & (DBG_USERDATA|DBG_PLUGIN|DBG_FILTER|DBG_CONFLICT))) {
+        level = DLT_LOG_DEBUG;
+      }
+      if (level < DLT_LOG_VERBOSE &&
+          (aDbgMask & DBG_DETAILS)) {
+        level = (DltLogLevelType)((int)level + 1);
+      }
+      if ((aTextSize > 0 && strlen(aText) > aTextSize) ||
+          strstr(aText, "&html;") ||
+          strstr(aText, "&sp;")) {
+            // Must make a copy and potentially filter out html markup.
+        string buffer;
+        buffer.reserve(aTextSize);
+        DbgText2PlainText(aText, aTextSize ? aTextSize : strlen(aText), buffer);
+        DLT_LOG(*context, level, DLT_STRING(buffer.c_str()));
+      } else {
+        // Fast path: log directly.
+        DLT_LOG(*context, level, DLT_STRING(aText));
+      }
+      return;
+    }
+#endif // USE_DLT
+
     // dissect into lines
     cAppCharP end=aTextSize ? aText+aTextSize : NULL;
     bool firstLine=true;
@@ -1323,11 +1472,47 @@ void TDebugLoggerBase::internalCloseBlocks(TDBG_LOCATION_PROTO cAppCharP aBlockN
   }
 } // TDebugLoggerBase::internalCloseBlocks
 
+#ifdef USE_DLT
+static void RegisterContext(DltContext *aHandle, const char *aContextID, const char *aDescription)
+{
+  std::string envName = "LIBSYNTHESIS_";
+  envName += aContextID;
+  const char *value = getenv(envName.c_str());
+  if (value) {
+    // Explicit level.
+    DltLogLevelType level = (DltLogLevelType)atoi(value);
+    dlt_register_context_ll_ts(aHandle, aContextID, aDescription, level, DLT_TRACE_STATUS_OFF);
+  } else {
+    // Default level.
+    dlt_register_context(aHandle, aContextID, aDescription);
+  }
+}
+#endif // USE_DLT
 
 // start debugging output if needed and sets fOutStarted
 bool TDebugLoggerBase::DebugStartOutput(void)
 {
   if (!fOutStarted) {
+#ifdef USE_DLT
+    if (fDbgOptionsP && fDbgOptionsP->fOutputFormat==dbgfmt_dlt) {
+      // Register our logging contexts.
+      if (!DbgDLTInitialized) {
+        RegisterContext(&DbgProtoContext, "PROT", "SyncML protocol related information");
+        RegisterContext(&DbgSessionContext, "SESS", "session management related information");
+        RegisterContext(&DbgAdminContext, "ADMN", "verything that has to do with administrative data (anchors, targets, map table)");
+        RegisterContext(&DbgDataContext, "DATA", "Everything that has to do with handling user data (data objects). Actual user data will however be shown only if loglevel >= debug.");
+        RegisterContext(&DbgRemoteInfoContext, "REMI", "This shows information delivered in the remote party's device information, such as manufacturer name, datatypes supported, fields supported etc.");
+        RegisterContext(&DbgParseContext, "PARS", "This shows information related to parsing and processing incoming data from the remote party. Actual user data will however be shown only if loglevel >= debug.");
+        RegisterContext(&DbgGenerateContext, "GEN", "This shows information related to generating outgoing data for the remote party. Actual user data will however be shown only if loglevel >= debug.");
+        RegisterContext(&DbgTranspContext, "TRNS", "shows transport (http and TCP communication) related information");
+        RegisterContext(&DbgSyncMLTKContext, "SMLT", "messages generated by the SyncML Toolkit code");
+        RegisterContext(&DbgDefaultContext, "SYS", "any other libsynthesis debug log message that does not fit in any of the other contexts");
+        DbgDLTInitialized = true;
+      }
+      fOutStarted = true;
+    }
+    else
+#endif // USE_DLT
     if (fOutputLoggerP) {
       // using another logger, call it to start output
       fOutStarted = fOutputLoggerP->DebugStartOutput();
@@ -1394,6 +1579,23 @@ void TDebugLoggerBase::DebugFinalizeOutput(void)
 // Output single line to debug channel (includes indenting and other prefixing, but no further formatting)
 void TDebugLoggerBase::DebugPutLine(TDBG_LOCATION_PROTO cAppCharP aText, stringSize aTextSize, bool aPre)
 {
+#ifdef USE_DLT
+  if (fDbgOptionsP && fDbgOptionsP->fOutputFormat==dbgfmt_dlt) {
+    // One example where this gets called is DebugVOpen/CloseBlock()
+    // with lines prepared as if we are printing plain text. Use
+    // a fairly neutral log level here.
+    if (aText) {
+      if (aTextSize > 0 && aTextSize < strlen(aText)) {
+        string buffer(aText, aTextSize);
+        DLT_LOG(DbgDefaultContext, DLT_LOG_INFO, DLT_STRING(buffer.c_str()));
+      } else {
+        DLT_LOG(DbgDefaultContext, DLT_LOG_INFO, DLT_STRING(aText));
+      }
+    }
+    return;
+  }
+#endif // USE_DLT
+
   if (!aText || (!fDbgOutP && !fOutputLoggerP)) return;
   if (*aText) {
     // not an empty line
@@ -1480,6 +1682,18 @@ TDebugLogger::~TDebugLogger()
 
 #ifdef MULTI_THREAD_SUPPORT
 
+void TDebugLogger::setOptions(const TDbgOptions *aDbgOptionsP)
+{
+  TDebugLoggerBase::setOptions(aDbgOptionsP);
+  TSubThreadLog* subThreadP = fSubThreadLogs;
+  while (subThreadP) {
+    if (subThreadP->fSubThreadLogger) {
+      subThreadP->fSubThreadLogger->setOptions(aDbgOptionsP);
+    }
+    subThreadP = subThreadP->fNext;
+  }
+}
+
 /// @brief find (and possibly delete) subthread record
 /// @param aAndRemove[in] if set, the subthread record will be removed in a thread safe way
 ///        IF AND ONLY IF aThreadID is the calling thread (i.e. only own thread may be removed from the list)!
@@ -1507,9 +1721,15 @@ TSubThreadLog *TDebugLogger::findSubThread(uInt32 aThreadID, bool aAndRemove)
 /// @brief find or create logger for subthread
 TDebugLoggerBase *TDebugLogger::getThreadLogger(bool aCreateNew)
 {
+  if (fOutputLoggerP) {
+    TDebugLoggerBase *logger = fOutputLoggerP->getThreadLogger(aCreateNew);
+    if (logger)
+      return logger;
+  }
+
   if (!fDbgOptionsP || fDbgOptionsP->fSubThreadMode==dbgsubthread_none)
     return this; // no options, do not handle subthreads specially
-  uInt32 threadID = myThreadID();
+  uIntArch threadID = myThreadID();
   if (fDbgOptionsP->fSubThreadMode==dbgsubthread_linemix || threadID==fMainThreadID) {
     // In line mix and for mainthread - I am the logger for this thread!
     return this;
@@ -1541,7 +1761,7 @@ TDebugLoggerBase *TDebugLogger::getThreadLogger(bool aCreateNew)
         // - create new base logger
         subThreadP->fSubThreadLogger = new TDebugLoggerBase(fGZonesP);
         // - install output (copy)
-        subThreadP->fSubThreadLogger->installOutput(fDbgOutP->clone());
+        subThreadP->fSubThreadLogger->installOutput(fDbgOutP ? fDbgOutP->clone() : NULL);
         // - same options
         subThreadP->fSubThreadLogger->setOptions(getOptions());
         // - inherit current mask/enable
@@ -1549,7 +1769,7 @@ TDebugLoggerBase *TDebugLogger::getThreadLogger(bool aCreateNew)
         subThreadP->fSubThreadLogger->setEnabled(fDebugEnabled);
         // - debug path is same as myself plus Thread ID
         subThreadP->fSubThreadLogger->setDebugPath(fDbgPath.c_str());
-        StringObjPrintf(s,"_%lu",threadID);
+        StringObjPrintf(s,"_%lu",(long unsigned)threadID);
         subThreadP->fSubThreadLogger->appendToDebugPath(s.c_str());
         break;
       case dbgsubthread_suppress:
@@ -1634,7 +1854,7 @@ void TDebugLogger::DebugShowSubThreadOutput(void)
 void TDebugLogger::DebugThreadOutputDone(bool aRemoveIt)
 {
   #ifdef MULTI_THREAD_SUPPORT
-  uInt32 threadID = myThreadID();
+  uIntArch threadID = myThreadID();
   if (threadID==fMainThreadID) {
     // current main thread done
     fMainThreadID = 0;
@@ -1664,7 +1884,7 @@ void TDebugLogger::DebugThreadOutputDone(bool aRemoveIt)
 void TDebugLogger::DebugDefineMainThread(void)
 {
   #ifdef MULTI_THREAD_SUPPORT
-  uInt32 threadID = myThreadID();
+  uIntArch threadID = myThreadID();
   // if this is already the main thread, no op
   if (threadID == fMainThreadID)
     return; // nop, done
