@@ -14,7 +14,7 @@
  *  is 1:1 used as datastore connector for the
  *  so called demo server/client.
  *
- *  Copyright (c) 2005-2009 by Synthesis AG (www.synthesis.ch)
+ *  Copyright (c) 2005-2011 by Synthesis AG + plan44.ch
  *
  *
  *  E X A M P L E    C O D E
@@ -493,6 +493,7 @@ TSyError Session_SaveNonce( CContext sContext, cAppCharP nonce )
   DEBUG_DB      ( sc->fCB, MyDB, Se_SN, "%d '%s'", sc,nonce );
 
   if (sc->fDev) { sc->fDev->fToken= nonce;
+                  sc->fDevList.fChanged= true;
     err=          sc->fDevList.SaveDB( true, sc->fCB ); // save it
   } // if
 
@@ -884,7 +885,7 @@ void ThreadMayChangeNow( CContext aContext )
   ContextP         ac= DBC( aContext );
   DEBUG_Exotic_DB( ac->fCB, MyDB,Da_TC, "(%08X)", ac );
 
-                   ac->fNewItem.SaveDB ( false ); // eventually partly save
+                   ac->fNewItem.SaveDB ( false ); // possibly partly save
                    ac->fItemList.SaveDB( false );
 } /* ThreadMayChangeNow */
 
@@ -933,30 +934,41 @@ void DispItems( CContext aContext, bool allFields, cAppCharP specificItem )
 #ifndef DISABLE_PLUGIN_DATASTOREADMIN
 /* -- ADMINISTRATION ------------------------------------------------------------ */
 /* (will be passed to the "admindata" module) */
-TSyError LoadAdminData( CContext aContext, cAppCharP aLocDB,
-                                           cAppCharP aRemDB, appCharP *adminData ) {
-  return DBC( aContext )->fAdmin.LoadAdminData     ( aLocDB,aRemDB,    adminData );
-} /* LoadAdminData */
+TSyError LoadAdminData     ( CContext    aContext,    cAppCharP    aLocDB,
+                                                      cAppCharP    aRemDB, appCharP  *adminData ) {
+  return DBC( aContext )->fAdmin.LoadAdminData                   ( aLocDB, aRemDB,    adminData );
+}     // LoadAdminData
+
+TSyError LoadAdminDataAsKey( CContext /* aContext */, cAppCharP /* aLocDB */,
+                                                      cAppCharP /* aRemDB */, KeyH /* adminKey */ ) {
+  return LOCERR_NOTIMP;
+}     // LoadAdminDataAsKey
+
+
+TSyError SaveAdminData    ( CContext     aContext, cAppCharP  adminData ) {
+  return DBC( aContext )->fAdmin.SaveAdminData              ( adminData );
+}     // SaveAdminData
+
+TSyError SaveAdminDataAsKey( CContext /* aContext */, KeyH /* adminKey */ ) {
+  return LOCERR_NOTIMP;
+}     // SaveAdminDataAsKey
 
-TSyError SaveAdminData( CContext aContext, cAppCharP adminData ) {
-  return DBC( aContext )->fAdmin.SaveAdminData     ( adminData );
-} /* SaveAdminData */
 
 bool   ReadNextMapItem( CContext aContext,  MapID mID, bool aFirst ) {
   return DBC( aContext )->fAdmin.ReadNextMapItem( mID,      aFirst );
-} /* ReadNextMapItem */
+}     // ReadNextMapItem
 
 TSyError InsertMapItem( CContext aContext, cMapID mID ) {
   return DBC( aContext )->fAdmin.InsertMapItem  ( mID );
-} /* InsertMapItem */
+}     // InsertMapItem
 
 TSyError UpdateMapItem( CContext aContext, cMapID mID ) {
   return DBC( aContext )->fAdmin.UpdateMapItem  ( mID );
-} /* UpdateMapItem */
+}     // UpdateMapItem
 
 TSyError DeleteMapItem( CContext aContext, cMapID mID ) {
   return DBC( aContext )->fAdmin.DeleteMapItem  ( mID );
-} /* DeleteMapItem */
+}     // DeleteMapItem
 #endif // DISABLE_PLUGIN_DATASTOREADMIN
 
 
@@ -1127,13 +1139,13 @@ TSyError InsertItem( CContext aContext, cAppCharP aItemData, ItemID newID )
   string   newItemID;
   TDBItem* act;
 
-  ItemID_Struct a; a.item  = "";
-                   a.parent= newID->parent; if (!a.parent) a.parent= "";
+  ItemID_Struct a; a.item  = (appCharP)"";
+                   a.parent= newID->parent; if (!a.parent) a.parent= (appCharP)"";
 
   TSyError err= ac->fItemList.CreateEmptyItem( &a, newItemID, act, ac->fNewID );
 
-  if (err) a.item= "???"; // undefined
-  else     a.item= (char*)newItemID.c_str();
+  if (err) a.item= (appCharP)"???"; // undefined
+  else     a.item= (appCharP)newItemID.c_str();
   string   s= ItemID_Info( &a );
 
   DEBUG_DB( ac->fCB, MyDB,Da_II, "%s '%s' err=%d", s.c_str(), aItemData, err );
